==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/syncml/inc/mail.php <==
<?php
class IMAP
{
	var $_imap;
	var $_server;
	var $_delimiter;
	var $_username;
	var $_password;
	var $_last_mailbox;
	var $_imap_data;
	
	function open($server,$port,$username,$password)
	{
		$attr='';
		if ($port==993)
			$attr.='/ssl/novalidate-cert';
 
		$port=$port?(':'.$port):'';
    if ($port==':110')
			$port.='/pop3/notls';
    
		$this->_server='{'.$server.$port.$attr.'}';

		$this->_imap=@imap_open($this->_server,$username,$password);
		if (!$this->_imap)
			return false;

		$list=imap_getmailboxes($this->_imap,$this->_server,'INBOX');
		if (!$list || !is_array($list))
			return false;

		$this->_delimiter=$list[0]->delimiter;
		$this->_username=$username;
		$this->_password=$password;
		
		return true;
	}
	
	function reopen()
	{
		if (!$this->_imap)
		  $this->_imap=@imap_open($this->_server,$this->_username,$this->_password);
	}

	function close()
	{
		if ($this->_imap)
			imap_close($this->_imap,CL_EXPUNGE);
  }

	function get_mailboxes()
	{
		$list=imap_getmailboxes($this->_imap,$this->_server,'*');
		if (!is_array($list))
			return false;
			
		$mailboxes=array();
		foreach($list as $mailbox)
		{
			$name=$this->decode_mailbox(preg_replace('/^{[^}]*}/','',$mailbox->name));
    	$mailboxes[]=$name;
		}
		return $mailboxes;
	}

	function open_mailbox($mailbox,$encoded=false)
	{
    if (!$encoded)
			$mailbox=$this->_server.$this->encode_mailbox($mailbox);

		if ($this->_last_mailbox!=$mailbox)
			if (!imap_reopen($this->_imap,$mailbox))
				return false;
			else
			  $this->_last_mailbox=$mailbox;

		return true;
	}		
	
	function get_items_id($mailbox,$sequence='1:*')
	{
    $this->open_mailbox($mailbox);
        
		if ($num=imap_num_msg($this->_imap))
		{
			$mails=$this->fetch_fast($sequence);
		  if (!$mails)
				return false;

			foreach($mails as $mail)
				$result[$mail->uid]=array('date'=>$mail->internaldate,'flags'=>$this->encode_flags($mail),'size'=>$mail->size);

			return $result;
		}
		return array();
	}
	
  function get_items($mailbox,$sequence='1:*')
	{
    $this->open_mailbox($mailbox);
        
		if ($num=imap_num_msg($this->_imap))
		{
			$mails=$this->fetch_overview($sequence);
      if (!$mails)
				return false;

			foreach($mails as $mail)
			{
        $result[$mail->uid]=array('date'=>$mail->internaldate,'flags'=>$this->encode_flags($mail),'size'=>$mail->size);

				if ($mail->message_id)
					$this->_imap_data[$mailbox]['message-id'][$mail->message_id]=$mail->uid;
				$this->_imap_data[$mailbox]['others'][$mail->uid]=$mail->from.$mail->to.$mail->subject.strtotime($mail->date);
			}
			return $result;
		}
		return array();
	}

  function fetch_fast($sequence)
	{
		return imap_fetch_fast($this->_imap,$sequence);
	}
	
	function fetch_overview($sequence,$fUID=0)
	{	  
		return imap_fetch_overview($this->_imap,$sequence,$fUID);
	}

	function get_header($mailbox,$uid)
	{
    $this->open_mailbox($mailbox);

		$headers=imap_fetchheader($this->_imap,$uid,FT_UID);

    if ($headers!==false)
      return $headers;
    else
			return false;
	}
	
	function get_body($mailbox,$uid)
	{
    $this->open_mailbox($mailbox);
    
    $body=imap_body($this->_imap,$uid,FT_UID);
    
    if ($body!==false)
      return $body;
		else
			return false;
	}
	
	function add_message($mailbox,$message,$verify)
	{
	  $mail=mailparse_msg_create();
		if (!mailparse_msg_parse($mail,$message))
		  return false;
		  
		$parts=mailparse_msg_get_structure($mail);
    $part=mailparse_msg_get_part($mail,$parts[0]);
		$info=mailparse_msg_get_part_data($part);
		$headers=$info['headers'];
		$message_id=$headers['message-id'];
		mailparse_msg_free($mail);

		if ($verify)
			if ($uid=$this->_imap_data[$mailbox]['message-id'][$message_id])
		  	return $uid;
		  else
				if ($uid=array_search($headers['from'].$headers['to'].$headers['subject'].strtotime($headers['date']),$this->_imap_data[$mailbox]['others']))
				  return $uid;

		$mailbox=$this->_server.$this->encode_mailbox($mailbox);
		$this->open_mailbox($mailbox,true);

		if (!imap_append($this->_imap,$mailbox,$message))
			return false;

		$new=imap_search($this->_imap,'RECENT');
		if ($new)
		{
			$sequence='';
			foreach($new as $item)
				$sequence.=($sequence?',':'').$item;

			$candidates=$this->fetch_fast($sequence);
			if (count($candidates)==1)
				return $candidates[0]->uid;
			else
				foreach($candidates as $item)
					if ($item->message_id==$message_id)
						return $item->uid;
		}
		return false;
	}

	function delete_message($mailbox,$uid)
	{
		$this->open_mailbox($mailbox);

		if (!imap_delete($this->_imap,$uid,FT_UID))
			return false;
			
		return true;
	}

	function set_flags($mailbox,$uid,$flags,$mask=127)
	{
		$this->open_mailbox($mailbox);

		$flags_set=array('\\Answered ','\\Deleted ','','\\Flagged ','','\\Seen ');
		$set_flags='';
		$unset_flags='';
		
		foreach($flags_set as $flag)
		{
			$flags_modulo=$flags%2;
			$mask_modulo=$mask%2;
			
			if ($mask_modulo)
				if ($flags_modulo)
			  	$set_flags.=$flag;
				else
			  	$unset_flags.=$flag;

			$flags=($flags-$flags_modulo)/2;
			$mask=($mask-$mask_modulo)/2;
		}

		if ($set_flags)
			if (!imap_setflag_full($this->_imap,$uid,$set_flags,ST_UID))
				return false;

		if ($unset_flags)
			if (!imap_clearflag_full($this->_imap,$uid,$unset_flags,ST_UID))
				return false;
			
		return true;
	}

	function encode_mailbox($mailbox)
	{
		if ($this->_delimiter!='/')
			$mailbox=str_replace('/',$this->_delimiter,$mailbox);

		return mb_convert_encoding($mailbox,'UTF7-IMAP','UTF-8');
	}

	function decode_mailbox($mailbox)
	{
		$mailbox=mb_convert_encoding($mailbox,'UTF-8','UTF7-IMAP');

		if ($this->_delimiter!='/')
			$mailbox=str_replace($this->_delimiter,'/',$mailbox);

		return $mailbox;
	}
	
	function encode_flags($flags_array)
	{
		$flags=0;
		foreach($flags_array as $key=>$value)
		  if ($value==1 || strtolower($value)=='true')
		    switch(strtoupper($key))
		    {
					case 'ANSWERED':
					case 'REPLIED':
					  $flags+=1;
						break;

					case 'DELETED':
					  $flags+=2;
						break;

			 /* case 'DRAFT':
					  $flags+=4;
						break; */

					case 'FLAGGED':
					  $flags+=8;
						break;

			 /* case 'RECENT':
					  $flags+=16;
						break; */

					case 'READ':
					case 'SEEN':
            $flags+=32;
						break;

					case 'FORWARDED':
            $flags+=64;
				}
		return $flags;
	}
	
	function encode_syncml($flags,$email_data)
	{
		$email=new xml_tree_node('Email',NULL);
		
		$flags_set=array('replied','deleted','','flagged','','read');
		
		foreach($flags_set as $flag)
		{
			$modulo=$flags%2;
			if ($flag)
      	$email->new_child_data($flag,NULL,$modulo?'true':'false');
			$flags=($flags-$modulo)/2;
		}

		$email->new_child_data('emailitem',NULL,'<![CDATA['.$email_data.']]>');
		
		return $email->print_node(0,false);
	}

	function decode_syncml($data)
	{
    $start=strpos($data,'<emailitem>');

		if ($start!==false)
		{
			$end=strpos($data,'</emailitem>');

			$flags=substr($data,0,$start).substr($data,$end+12);
			$mail=substr($data,$start+11,$end-$start-11);

			$start=strpos($mail,'<![CDATA[');
			$end=strpos($mail,']]');
			$mail=substr($mail,$start+9,$end-$start-9);
			$result['mail']=$mail;
		}
		else
		  $flags=$data;

		$email=text_to_xml($flags);

		$children=&$email->children();
		$flags=array();
		
		for($i=0;$i<$email->children_count();$i++)
		{
    	$tag=&$children[$i];
			$name=$tag->get_name();
			$flags[$name]=$email->get_data($name);
		}

		$result['flags']=$this->encode_flags($flags);

		return $result;
	}
	
	function encode_plain($folder,$flags,$data)
	{
    $mail=mailparse_msg_create();
		if (!mailparse_msg_parse($mail,$data))
		  return false;

		$parts=mailparse_msg_get_structure($mail);
    $part=mailparse_msg_get_part($mail,$parts[0]);
		$info=mailparse_msg_get_part_data($part);
		$headers=$info['headers'];
		mailparse_msg_free($mail);
		
		//Podpora vce flag
		//More flags support
		/*
		$flags_set=array('replied','deleted','','flagged','','read','forwarded');
		foreach($flags_set as $flag)
		{
			$modulo=$flags%2;
			if ($flag)
				$headers['x-sync-message-'.$flag]=$modulo?'true':'false';
				
  		$flags=($flags-$modulo)/2;
		}
		*/
		$headers['x-sync-message-read']=($flags&32)?'true':'false';
		$headers['x-sync-parent-folder']=strtolower($folder);
		
		$header='';
		foreach($headers as $key=>$value)
		{
			$case=true;
			for($i=0;$i<strlen($key);$i++)
			{
				$header.=$case?strtoupper($key[$i]):$key[$i];
				if ($key[$i]=='-')
				  $case=true;
				else
				  $case=false;
			}
      $header.=': '.$value."\r\n";
		}
		$pos=strpos($data,"\r\n\r\n");
		if ($pos!==false)
		  return $header.substr($data,$pos+2);
		else
		  return $header."\r\n\r\n";
	}
	
	function decode_plain($data,$remote_id)
	{
	  $mail=mailparse_msg_create();
		if (!mailparse_msg_parse($mail,$data))
		  return false;

		$parts=mailparse_msg_get_structure($mail);
    $part=mailparse_msg_get_part($mail,$parts[0]);
		$info=mailparse_msg_get_part_data($part);
		$headers=$info['headers'];
		mailparse_msg_free($mail);

		//Podpora vce flag
		//More flags support
		/*
		foreach($headers as $header=>$value)
		  if (strpos($header,'x-sync-message-')!==false)
		    $flags[substr($header,15)]=$value;
		*/
		$flags['read']=$headers['x-sync-message-read'];
		$result['flags']=$this->encode_flags($flags);
		
		if ($headers['message-id'])
		  $result['mail']=$data;
		else
		  $result['mail']='Message-ID: '.$remote_id."\r\n".$data;

		return $result;
	}
	
	function extract_attach($mailbox,$uid)
	{
    $this->open_mailbox($mailbox);
		$struct=imap_fetchstructure($this->_imap,$uid,FT_UID);
		if ($struct===false)
		  return false;

    $parts=$struct->parts;

    if (!$parts)
      $content=imap_body($this->_imap,$uid,FT_UID|FT_PEEK);
		else
		{
			$endwhile=false;
			$i=0;
      $stack=array(); 
      $content='';
      
      while(!$endwhile)
			{
        if (!$parts[$i])
          if (count($stack)>0)
					{
            $parts=$stack[count($stack)-1]['p'];
            $i=$stack[count($stack)-1]['i']+1;
            array_pop($stack);
          }
					else
            $endwhile=true;

        if (!$endwhile)
        {
          if (strtoupper($parts[$i]->disposition)!="ATTACHMENT" && strtoupper($parts[$i]->subtype)=="PLAIN")
					{
            $partstring="";
            foreach($stack as $s)
              $partstring.=($s['i']+1).'.';

            $partstring.=($i+1);

						$subcontent=imap_fetchbody($this->_imap,$uid,$partstring,FT_UID|FT_PEEK);
						if ($subcontent!==false)
							$content.=$subcontent;
						else
						  return false;
					}
        	if ($parts[$i]->parts)
					{
          	$stack[]=array('p' =>$parts,'i'=>$i);
          	$parts=$parts[$i]->parts;
          	$i=0;
        	}
					else
          	$i++;
        }
      } 
    } 
		return $content;
	}
}
?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/syncml/inc/mail.php <==
<?php
class IMAP
{
	var $_imap;
	var $_server;
	var $_delimiter;
	var $_username;
	var $_password;
	var $_last_mailbox;
	var $_imap_data;
	
	function open($server,$port,$username,$password)
	{
		//IMAP SSL
		if ($port==993) 
			$attr='/ssl/novalidate-cert'; 
		else
 			//POP3
      if ($port=='110')
				$attr='/pop3/notls'; 
			else
 				//POP3 SSL
        if ($port=='995') 
					$attr='/pop3/ssl/novalidate-cert'; 
				else
	        //IMAP plain
          $attr='/notls';
    
		$this->_server='{'.$server.($port?(':'.$port):'').$attr.'}';

		$this->_imap=@imap_open($this->_server,$username,$password);
		
		if (!$this->_imap)
			return false;

		$list=imap_getmailboxes($this->_imap,$this->_server,'INBOX');
		if (!$list || !is_array($list))
			return false;

		$this->_delimiter=$list[0]->delimiter;
		$this->_username=$username;
		$this->_password=$password;
		
		return true;
	}
	
	function reopen()
	{
		if (!$this->_imap)
		  $this->_imap=@imap_open($this->_server,$this->_username,$this->_password);
	}

	function close()
	{
		if ($this->_imap)
			imap_close($this->_imap,CL_EXPUNGE);
  }

	function get_mailboxes()
	{
		$list=imap_getmailboxes($this->_imap,$this->_server,'*');
	
		if (!is_array($list))
			return false;
			
		$mailboxes=array();
		foreach($list as $mailbox)
		{
			$name=$this->decode_mailbox(preg_replace('/^{[^}]*}/','',$mailbox->name));
    	$mailboxes[]=$name;
		}
		return $mailboxes;
	}

	function open_mailbox($mailbox,$encoded=false)
	{
    if (!$encoded)
			$mailbox=$this->_server.$this->encode_mailbox($mailbox);

		if ($this->_last_mailbox!=$mailbox)
			if (!imap_reopen($this->_imap,$mailbox))
				return false;
			else
			  $this->_last_mailbox=$mailbox;

		return true;
	}		
	
	function get_items_id($mailbox,$sequence='1:*')
	{
    $this->open_mailbox($mailbox);
        
		if ($num=imap_num_msg($this->_imap))
		{
			$mails=$this->fetch_fast($sequence);
		  if (!$mails)
				return false;

			foreach($mails as $mail)
				$result[$mail->uid]=array('date'=>$mail->internaldate,'flags'=>$this->encode_flags($mail),'size'=>$mail->size);

			return $result;
		}
		return array();
	}
	
  function get_items($mailbox,$sequence='1:*')
	{
    $this->open_mailbox($mailbox);
        
		if ($num=imap_num_msg($this->_imap))
		{
			$mails=$this->fetch_overview($sequence);
      if (!$mails)
				return false;

			foreach($mails as $mail)
			{
        $result[$mail->uid]=array('date'=>$mail->internaldate,'flags'=>$this->encode_flags($mail),'size'=>$mail->size);

				if ($mail->message_id)
					$this->_imap_data[$mailbox]['message-id'][$mail->message_id]=$mail->uid;
				$this->_imap_data[$mailbox]['others'][$mail->uid]=$mail->from.$mail->to.$mail->subject.strtotime($mail->date);
			}
			return $result;
		}
		return array();
	}

  function fetch_fast($sequence)
	{
		return imap_fetch_fast($this->_imap,$sequence);
	}
	
	function fetch_overview($sequence,$fUID=0)
	{	  
		return imap_fetch_overview($this->_imap,$sequence,$fUID);
	}

	function get_header($mailbox,$uid)
	{
    $this->open_mailbox($mailbox);

		$headers=imap_fetchheader($this->_imap,$uid,FT_UID);

    if ($headers!==false)
      return $headers;
    else
			return false;
	}
	
	function get_body($mailbox,$uid)
	{
    $this->open_mailbox($mailbox);
    
    $body=imap_body($this->_imap,$uid,FT_UID);
    
    if ($body!==false)
      return $body;
		else
			return false;
	}
	
	function parse_headers($message)
	{
    $mail=mailparse_msg_create();
		if (!mailparse_msg_parse($mail,$message))
		  return false;

		$parts=mailparse_msg_get_structure($mail);
    $part=mailparse_msg_get_part($mail,$parts[0]);
		$info=mailparse_msg_get_part_data($part);
		mailparse_msg_free($mail);
		
		$headers=&$info['headers'];
	
		foreach($headers as $key=>$header)
			if (is_array($header))
			{	
				foreach($header as $index=>$value)
					if (trim($value)==='')
						unset($header[$index]);	
			
				$headers[$key]=implode(',',$header);
			}
			
		return $headers;
	}
	
	function add_message($mailbox,$message,$verify)
	{
	  $headers=$this->parse_headers($message);
		if ($headers===false)
		  return false;
		  
		$message_id=$headers['message-id'];

		if ($verify)
			if ($uid=$this->_imap_data[$mailbox]['message-id'][$message_id])
		  	return $uid;
		  else
				if ($uid=array_search($headers['from'].$headers['to'].$headers['subject'].strtotime($headers['date']),$this->_imap_data[$mailbox]['others']))
				  return $uid;

		$mailbox=$this->_server.$this->encode_mailbox($mailbox);
		$this->open_mailbox($mailbox,true);

		if (!imap_append($this->_imap,$mailbox,$message))
			return false;

		$new=imap_search($this->_imap,'RECENT');
		if ($new)
		{
			$sequence='';
			foreach($new as $item)
				$sequence.=($sequence?',':'').$item;

			$candidates=$this->fetch_fast($sequence);
			if (count($candidates)==1)
				return $candidates[0]->uid;
			else
				foreach($candidates as $item)
					if ($item->message_id==$message_id)
						return $item->uid;
		}
		return false;
	}

	function delete_message($mailbox,$uid)
	{
		$this->open_mailbox($mailbox);

		if (!imap_delete($this->_imap,$uid,FT_UID))
			return false;
			
		return true;
	}

	function set_flags($mailbox,$uid,$flags,$mask=127)
	{
		$this->open_mailbox($mailbox);

		$flags_set=array('\\Answered ','\\Deleted ','','\\Flagged ','','\\Seen ');
		$set_flags='';
		$unset_flags='';
		
		foreach($flags_set as $flag)
		{
			$flags_modulo=$flags%2;
			$mask_modulo=$mask%2;
			
			if ($mask_modulo)
				if ($flags_modulo)
			  	$set_flags.=$flag;
				else
			  	$unset_flags.=$flag;

			$flags=($flags-$flags_modulo)/2;
			$mask=($mask-$mask_modulo)/2;
		}

		if ($set_flags)
			if (!imap_setflag_full($this->_imap,$uid,$set_flags,ST_UID))
				return false;

		if ($unset_flags)
			if (!imap_clearflag_full($this->_imap,$uid,$unset_flags,ST_UID))
				return false;
			
		return true;
	}

	function encode_mailbox($mailbox)
	{
		if ($this->_delimiter!='/')
			$mailbox=str_replace('/',$this->_delimiter,$mailbox);

		return mb_convert_encoding($mailbox,'UTF7-IMAP','UTF-8');
	}

	function decode_mailbox($mailbox)
	{
		$mailbox=mb_convert_encoding($mailbox,'UTF-8','UTF7-IMAP');

		if ($this->_delimiter!='/')
			$mailbox=str_replace($this->_delimiter,'/',$mailbox);

		return $mailbox;
	}
	
	function encode_flags($flags_array)
	{
		$flags=0;
		foreach($flags_array as $key=>$value)
		  if ($value==1 || strtolower($value)=='true')
		    switch(strtoupper($key))
		    {
					case 'ANSWERED':
					case 'REPLIED':
					  $flags+=1;
						break;

					case 'DELETED':
					  $flags+=2;
						break;

			 /* case 'DRAFT':
					  $flags+=4;
						break; */

					case 'FLAGGED':
					  $flags+=8;
						break;

			 /* case 'RECENT':
					  $flags+=16;
						break; */

					case 'READ':
					case 'SEEN':
            $flags+=32;
						break;

					case 'FORWARDED':
            $flags+=64;
				}
		return $flags;
	}
	
	function encode_syncml($flags,$email_data)
	{
		$email=new xml_tree_node('Email',NULL);
		
		$flags_set=array('replied','deleted','','flagged','','read');
		
		foreach($flags_set as $flag)
		{
			$modulo=$flags%2;
			if ($flag)
      	$email->new_child_data($flag,NULL,$modulo?'true':'false');
			$flags=($flags-$modulo)/2;
		}

		$email->new_child_data('emailitem',NULL,'<![CDATA['.$email_data.']]>');
		
		return $email->print_node(0,false);
	}

	function decode_syncml($data)
	{
    $start=strpos($data,'<emailitem>');

		if ($start!==false)
		{
			$end=strpos($data,'</emailitem>');

			$flags=substr($data,0,$start).substr($data,$end+12);
			$mail=substr($data,$start+11,$end-$start-11);

			$start=strpos($mail,'<![CDATA[');
			$end=strpos($mail,']]');
			$mail=substr($mail,$start+9,$end-$start-9);
			$result['mail']=$mail;
		}
		else
		  $flags=$data;

		$email=text_to_xml($flags);

		$children=&$email->children();
		$flags=array();
		
		for($i=0;$i<$email->children_count();$i++)
		{
    	$tag=&$children[$i];
			$name=$tag->get_name();
			$flags[$name]=$email->get_data($name);
		}

		$result['flags']=$this->encode_flags($flags);

		return $result;
	}
	
	function create_header($headers)
	{
    $header='';
		foreach($headers as $key=>$value)
		{
			$case=true;
			for($i=0;$i<strlen($key);$i++)
			{
				$header.=$case?strtoupper($key[$i]):$key[$i];
				if ($key[$i]=='-')
				  $case=true;
				else
				  $case=false;
			}
      $header.=': '.$value."\r\n";
		}
		
		return $header."\r\n";
	}

	function encode_plain($folder,$flags,$data)
	{
    $headers=$this->parse_headers($data);
		if ($headers===false)
		  return false;

		//Podpora vce flag
		//More flags support
		/*
		$flags_set=array('replied','deleted','','flagged','','read','forwarded');
		foreach($flags_set as $flag)
		{
			$modulo=$flags%2;
			if ($flag)
				$headers['x-sync-message-'.$flag]=$modulo?'true':'false';
				
  		$flags=($flags-$modulo)/2;
		}
		*/
		
		$headers['x-sync-message-read']=($flags&32)?'true':'false';
		$headers['x-sync-parent-folder']=strtolower($folder);
		
		$header=$this->create_header($headers);
		
		$pos=strpos($data,"\r\n\r\n");
		if ($pos!==false)
		  return $header.substr($data,$pos+4);
		else
		  return $header;
	}
	
	function decode_plain($data,$remote_id)
	{
    $headers=$this->parse_headers($data);
		if ($headers===false)
		  return false;

		//Podpora vce flag
		//More flags support
		/*
		foreach($headers as $header=>$value)
		  if (strpos($header,'x-sync-message-')!==false)
		    $flags[substr($header,15)]=$value;
		*/
		$flags['read']=$headers['x-sync-message-read'];
		$result['flags']=$this->encode_flags($flags);
		
		if ($headers['message-id'])
		  $result['mail']=$data;
		else
		  $result['mail']='Message-ID: '.$remote_id."\r\n".$data;

		return $result;
	}
	
	function decode_sms($data)
	{
		$attach=iconv('ucs-2','utf-8',$data);

		$start=strpos($attach,'BEGIN:VBODY');
		if ($start!==false)
		{
			$end=strpos($attach,'END:VBODY');
			$mail=substr($attach,$start+13,$end-$start-13);
		}

		$result['mail']=$mail;
	}
	
	function extract_attach($mailbox,$uid)
	{
    $header=$this->get_header($mailbox,$uid);
		if ($header===false)
			return false;
    
		$struct=imap_fetchstructure($this->_imap,$uid,FT_UID);
		if ($struct===false)
		  return false;

    $parts=$struct->parts;

    if (!$parts)
      $content=imap_body($this->_imap,$uid,FT_UID|FT_PEEK);
		else
		{
			$headers=$this->parse_headers($header);
			if ($headers===false)
		  	return false;
			
			$endwhile=false;
			$i=0;
      $stack=array(); 
      $content='';
      
      while(!$endwhile)
			{
        if (!$parts[$i])
          if (count($stack)>0)
					{
            $parts=$stack[count($stack)-1]['p'];
            $i=$stack[count($stack)-1]['i']+1;
            array_pop($stack);
          }
					else
            $endwhile=true;

        if (!$endwhile)
        {
          if (strtoupper($parts[$i]->disposition)!='ATTACHMENT' && strtoupper($parts[$i]->subtype)=='PLAIN')
					{
            $partstring="";
            foreach($stack as $s)
              $partstring.=($s['i']+1).'.';

            $partstring.=($i+1);

						$subcontent=imap_fetchbody($this->_imap,$uid,$partstring,FT_UID|FT_PEEK);
						if ($subcontent!==false)
							$content.=$subcontent;
						else
						  return false;
					}
        	if ($parts[$i]->parts)
					{
          	$stack[]=array('p' =>$parts,'i'=>$i);
          	$parts=$parts[$i]->parts;
          	$i=0;
        	}
					else
          	$i++;
        }
      }
			
			$headers['content-type']=preg_replace('/^([^;]{0,})/i','text/plain',$headers['content-type']);
			$header=$this->create_header($headers); 
    }		

		return array('header'=>$header,'body'=>$content);
	}
}
?>
